#include "sht71.h"

unsigned char write_byte_sht (unsigned char value) {
	unsigned char i = 0;
	unsigned char error = 0;
	for (i=0x80; i>0; i/=2) {
		if (i & value)
			set_DATA;
		if (!(i & value))
			clear_DATA;
		set_SCK;
		short_delay;
		clear_SCK;
	}
	set_DATA;
	set_SCK;
	if (PIN_SHT & (1<<P_DATA))
		error = 1;
	clear_SCK;
	return error;
}

unsigned char read_byte_sht (unsigned char ack) {
	unsigned char i = 0;
	unsigned char value = 0;
	set_DATA;
	for (i=0x80; i>0; i/=2) {
		set_SCK;
		if (PIN_SHT & (1<<P_DATA))
			value |= i;
		clear_SCK;
	}
	if (ack)
		clear_DATA;
	set_SCK;
	short_delay;
	clear_SCK;
	set_DATA;
	return value;
}

void start_comm_sht (void) {
	set_DATA;
	clear_SCK;
	short_delay;
	set_SCK;
	short_delay;
	clear_DATA;
	short_delay;
	clear_SCK;
	short_delay;
	set_SCK;
	short_delay;
	set_DATA;
	short_delay;
	clear_SCK;
}

void connection_reset_sht (void) {
	unsigned char i = 0;
	set_DATA;
	clear_SCK;
	for (i=0; i<9; i++) {
		set_SCK;
		clear_SCK;
	}
	start_comm_sht ();
}

unsigned char soft_reset_sht (void) {
	unsigned char error = 0;
	connection_reset_sht ();
	if (write_byte_sht (soft_reset))
		error = 1;
	return error;
}

double calc_dewpoint_sht (double temp, double hum) {
	double logEx = 0;
	double dewpoint = 0;
	logEx = 0.66077 + (7.5 * temp / (237.3 + temp)) + (log10 (hum) -2);
	dewpoint = ((0.66077 - logEx) * 237.3) / (logEx - 8.16077);
	return dewpoint;
}

double measure_temperature_sht (void) {
	connection_reset_sht ();
	soft_reset_sht ();
	start_comm_sht ();
	write_byte_sht (measure_temp);
	
	while (PIN_SHT & (1<<P_DATA));
	
	unsigned short temp = 0;
	temp |= read_byte_sht (1) << 8;
	temp |= read_byte_sht (1);
	
	double temperature = 0;
	temperature = ((double) temp)*0.01 - 40.0;
	
	return temperature;
}

double measure_humidity_sht (void) {
	connection_reset_sht ();
	soft_reset_sht ();
	start_comm_sht ();
	write_byte_sht (measure_humi);
	
	while (PIN_SHT & (1<<P_DATA));
	
	unsigned short humi = 0;
	humi |= read_byte_sht (1) << 8;
	humi |= read_byte_sht (1);
	
	double humidity = 0;
	humidity = -4.0 + 0.0405 * ((double) humi) - 0.0000028 * ((double) humi);
	return humidity;
}

double measure_true_humidity_sht (void) {
	double temp = measure_temperature_sht ();
	
	connection_reset_sht ();
	soft_reset_sht ();
	start_comm_sht ();
	write_byte_sht (measure_humi);
	
	while (PIN_SHT & (1<<P_DATA));
	
	unsigned short humi = 0;
	humi |= read_byte_sht (1) << 8;
	humi |= read_byte_sht (1);
	
	double true_humidity = 0;
	true_humidity = (temp - 25) * (0.01 + 0.00008 * (double) humi) + measure_humidity_sht ();

	return true_humidity;
}
